%{ 

(*
Authors: Brody Berg, Max Czapanskiy after Edwards
Date: Fall 2008
*)

open Ast 

let parse_error s = (* Called by the parser function on error *)
  print_endline s;
  flush stdout

%}

%token PLUS
%token MINUS
%token TIMES
%token DIVIDE
%token MOD
%token COLON 
%token SEMI
%token COMMA
%token ARROW
%token END
%token ASSIGN
%token PRINT
%token INTTOSTRING
%token LPAREN
%token RPAREN
%token QUOTE
%token LT GT EQ
%token QUESTION
%token <int> INT
%token <string> STRING
%token <string> ID
%token EOF

%start program
%type <Ast.program> program

%%

/* A program is a start state and a list of 
   states */ 
program: 
  /* */ { [] }
  | program state { ($2 :: $1 ) }

/* need to implement the ability to handle parameters */
state: 
   ID LPAREN arg_list RPAREN statement_list transition_list END 
        { { sname = $1;
            formals = $3;
            statements = $5;
            transitions =  $6; } }

/* An arg_list is a list of comma separated statements */
arg_list:
   /*  */                    { [] }
 | statement                 { [$1] }
 | statement COMMA arg_list  { $1::$3 }

/* A list of statements, starting a state the list may be empty */ 
statement_list:
  /* */ { [] }
  | statement statement_list { $1::$2 }   

/* A list of transitions within a state the list may be empty */  
transition_list: 
  /* */ { [] }
  | transition transition_list { $1::$2 } 

transition:
  QUESTION statement COLON statement_list ARROW call SEMI 
  { { condition = $2;
      actions = $4;
      target = $6; } } 

call:
   /* */ { { state = ""; args = []; } }
 | ID arg_list { { state = $1; args = $2; } }

statement:
    /* Nothing */                           { Empty }
  | INT                                     { Int($1) }
  | STRING                                  { String($1) }
  | ID			                    { Id($1) }
  | PRINT LPAREN statement RPAREN           { LibCallP($3) }
  | INTTOSTRING LPAREN statement RPAREN     { LibCallIS($3) }
  | statement ASSIGN statement              { Binop($1, Assign, $3) }
  | statement PLUS statement                { Binop($1, Add, $3) }
  | statement MINUS statement               { Binop($1, Sub, $3) }
  | statement TIMES statement               { Binop($1, Mult, $3) }
  | statement DIVIDE statement              { Binop($1, Div, $3) }
  | statement MOD statement                 { Binop($1, Mod, $3) }
  | statement LT statement                  { Binop($1, Less, $3) }
  | statement GT statement                  { Binop($1, Greater, $3) }
  | statement EQ statement                  { Binop($1, Equal, $3) } 
  
